// PDEsim.h: interface for the PDEsim class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PDESIM_H__756EBA21_C8B6_11D1_B6C3_00A0C995F22C__INCLUDED_)
#define AFX_PDESIM_H__756EBA21_C8B6_11D1_B6C3_00A0C995F22C__INCLUDED_

#include "Rand.h"	// Added by ClassView
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

typedef double myComplex[2];

class PDEsim  
{
public:
	void Restart();
	void SetRandom();
	double Sweep();
	PDEsim();
	virtual ~PDEsim();
	int GetDim(int i) const {return dim[i];}
	double GetPsi(int i, int j) const {return psi[i*dim[0]+j][0];}
	double GetDislocationSeparation() const {return dislocationSeparation;}
	double GetDt() const {return dt;}
	double GetKx() const {return kx;}
	double GetKy() const {return ky;}
	double GetNoise() const {return noise;}
	int GetNu() const {return nu;}
	int GetSeed() const {return seed;}
	double GetMin() const {return min;}
	double GetMax() const {return max;}
	void SetDislocationSeparation(double dislocationSeparation) 
		{this->dislocationSeparation = dislocationSeparation;}
	void SetDt(double dt) {this->dt = dt;}
	void SetKx(double kx) {this->kx = kx;}
	void SetKy(double ky) {this->ky = ky;}
	void SetNoise(double noise) {this->noise = noise;}
	void SetNu(int nu) {this->nu = nu; dim[0]=dim[1]=(1<<nu);} // 2^nu 
	void SetSeed(int seed) {this->seed = seed;}
	void SetMin(double min) {this->min = min;}
	void SetMax(double max) {this->max = max;}
// Swift-Hohenberg
	double GetEpsilon() const {return epsilon;}
	double GetK() const {return k;}
	void SetEpsilon(double epsilon) {this->epsilon = epsilon;}
	void SetK(double k) {this->k = k;}

// Extended Swift-Hohenberg
/*
	void SetC2(double c2) {this->c2 = c2;}
	void SetG_m(double g_m) {this->g_m = g_m;}
	double GetC2() const {return c2;}
	double GetG_m() const {return g_m;}
*/


protected:
	double dislocationSeparation;
	double dt;
	double kx, ky;
	double noise;
	double min;
	double max;
	int nu;	// 2^nu = dim[0] = dim[1] = size of simulation
	int seed;
	double D;
	int dim[2];
	Rand rand;
	double (*psi)[2];
	double * expLdt;

// Swift-Hohenberg
	void ComputeNk();
	double epsilon;
	double k;
	double (*Nk)[2];
	double (*NkOld)[2];
	double * expNonLinear;

// Generalized Swift-Hohenberg
/*	void ComputeZetas();
	double g_m;
	double c2;
	double (*zeta)[2];
	// Internal computations GSH
	double (*dZetaDx)[2];
	double (*dZetaDy)[2];
	double (*dDel2PsiDx)[2];
	double (*dDel2PsiDy)[2];
	double (*dPsiDx)[2];
	double (*dPsiDy)[2];
	double * meanLin;
*/
};

#endif // !defined(AFX_PDESIM_H__756EBA21_C8B6_11D1_B6C3_00A0C995F22C__INCLUDED_)
