/* (C) Copyright 1993 by Steven Trainoff.  Permission is granted to make
* any use of this code subject to the condition that all copies contain
* this notice and an indication of what has been changed.
*/
#include <stdio.h>
#include <math.h>
#include "spt.h"
/* #include "parseline.h" */

#ifndef REAL
#define REAL double		/* Precision of data */
#endif

#include "fft.h"


/* This program computes the degrees of freedom paramter of a range of fourier space.
 * The degree of freedom is defined as sum(power)^2/sum(power^2)
 * This has the property of measuring the "width" of a peak in fourier space.  If we have a sine
 * way dof=1 (or 2 if a windowing function has been applied).  If we have white noise dof=Nbins
 * The result is independant of the amplitude of the peak
 *
 * The routine can be used in two ways.  You can either define the range of peaks in fourier
 * space be setting the begin and end bins from the command line.  Or you can specify a width
 * of the region to be considered.  The program finds the largest peak and performs the
 * dof about that peak.
 */

extern char *progname;
static char dowindow=FALSE;	/* Should we apply a windowing function ? */
static char remove_mean=FALSE;  /* Should we remove the mean ? */
static int beginfreq = 0;
static int endfreq = MAXINT;
static int nbin = 0;

static CMDPARSE switches[] = {
	{ "beginfreq", 	INT,		&beginfreq}, 
	{ "begin",		INT,		&beginfreq}, 
	{ "b",			INT,		&beginfreq}, 
        { "endfreq",	INT,   	&endfreq}, 
        { "end",		INT,   	&endfreq}, 
        { "e",		INT,   	&endfreq}, 
	{ "mean",		FLAG,	&remove_mean },
	{ "m",		FLAG,	&remove_mean },
	{ "nbin",		INT,		&nbin },
	{ "n",			INT,		&nbin },
	{ "window",	FLAG,	&dowindow },
	{ "w",		FLAG,	&dowindow },
	{ "",			NONE,	NULL },				/* End of switches */
};


void main(argc, argv)
int argc;
char **argv;
{
	REAL *array;
	double mean;
	int i, arraysize;
	int n;
	double sumpower, sumsqrpower;

	progname = argv[0];

	if (parseline(&argc, &argv, switches) || argc != 1) {
		fprintf(stderr, "Usage: %s [-(w)indow] [-(m)ean][-(n)bin=0] n\n", progname);
		fprintf(stderr, "or %s [-(w)indow] [-(m)ean] [-(b)eginfreq] [-(e)ndfreq] n\n", progname);
		exit(-1);
	}

	n = atoi(argv[0]);
	arraysize = 1 << (ilog2(n-1) + 1);	/* Nearest power of two */

	/* Sanitity check the input parameters */
	array = (REAL *)emalloc(arraysize*sizeof(REAL));
	
	for (i = 0, mean = 0.; i < n; i++) {
		double temp;
		scanf("%lf", &temp);
		array[i] = temp;
		mean += temp;
	}

	/* Pad the rest of the array with 0 */
	for (; i < arraysize; i++)
		array[i] = 0.;

	/* Remove the mean */
	mean /= n;
	for (i = 0; i < n; i++)
		array[i] -= mean;

	if (dowindow)
		for (i = 0; i < n; i++)
			array[i] *= (1.0 - cos(2*PI*i/n))/2.;

	realfftmag(array, arraysize);

	if (! remove_mean) 		/* Add back the mean */
		array[0] = mean;
	
	if (nbin != 0) {	/* ie If we are asking for the program to find the biggest peak */
	   	int i, maxbin=0;
		double maxval=0.0, temp;
		
		for (i = 0; i < arraysize/2; i++) {
			if ((temp = fabs(array[i])) > maxval) {
				maxval = temp;
				maxbin = i;
			}
		}
		
		beginfreq = maxbin-nbin/2;
		endfreq = maxbin+nbin/2;
	}
				
	beginfreq = max(beginfreq, 0);
	endfreq = min(endfreq, arraysize/2);
	for (i=beginfreq, sumpower=0., sumsqrpower=0.; i <= endfreq; i++) {
	    double temp=sqr(array[i]);
	    sumpower += temp;
	    sumsqrpower += sqr(temp);
	}
	printf("%9.6e\n", sqr(sumpower)/sumsqrpower );
}

